#!/usr/bin/env python

import tempfile
import os
import sys
import errno
import subprocess
import config as configlib
import utilities as util
import options

appName = 'korporat'
defaultEntryPoint = 'menu'

config = None
options = options.Options()

def mkdir_p(path):
	if not os.path.exists(path):
		os.makedirs(path)
	# try:
	# 	os.makedirs(path)
	# except OSError:
	# 	_, err, _ = sys.exc_info()
	# 	if err.errno == errno.EEXIST and os.path.isdir(path):
	# 		pass
	# 	else:
	# 		raise

def res_path(path):
	return os.path.abspath(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'res', path))

def get_conf(key):
	v = config.get(key)
	if v == None:
		return ''
	return v

def get_app_conf(key):
	v = config.getAppConfig(appName, key)
	if v == None:
		return ''
	return v

def create_configtxt(path):
	confData = None
	if config.appData.has_key(appName):
		confData = config.appData[appName]

	confFileName = os.path.join(path)
	f = open(confFileName, 'w')

	# DB.PATH is always in e:
	f.write("DB.PATH,e:\\\r\n")

	if confData:
		for key in confData:
			if key == 'DB.PATH':
				continue
			val = confData[key]
			if type(val) is str or unicode:
				f.write("%s,%s\r\n" % (key, val))
			if type(val) is int:
				f.write("%s,,%d\r\n" % (key, val))
			if type(val) is float:
				f.write("%s,,,,%f\r\n" % (key, val))
			if type(val) is bool:
				bs = 'F'
				if val:
					bs = 'T'
				f.write("%s,,,%s\r\n" % (key, bs))

	f.flush()
	f.close()

	return

def create_runbat(path):
	batchFilePath = path
	batchFile = open(batchFilePath, 'w')
	batchFile.write('cd %s\r\n' % appName)

	batchFile.write('c:\\foxrun\\foxr')

	if options.appArgs and len(options.appArgs) > 0:
		for a in options.appArgs:
			batchFile.write(' ')
			batchFile.write(a)
	else:
		batchFile.write(' menu')

	batchFile.write("\r\n")
	batchFile.close()
	return


def launch():
	# Check 64bit
	is64bits = os.environ['DOSEMU_ARCH'] == '64'
	# Use appropriate dosemu
	dosemures = res_path('dosemu')
	if is64bits:
		dosemures = res_path('dosemu64')

	# Create working directory
	workDir = tempfile.mkdtemp()

	# Create drives
	drivesDir = os.path.join(workDir, 'drives')
	mkdir_p(drivesDir)

	# Drive C
	driveC = os.path.join(drivesDir, 'c')
	mkdir_p(driveC)
	# 1. autoexec.bat
	os.symlink(res_path('autoexec.bat'), os.path.join(driveC, 'autoexec.bat'))
	# 2. config.sys
	os.symlink(res_path('config.sys'), os.path.join(driveC, 'config.sys'))
	# 3. kernel.sys
	os.symlink(os.path.join(dosemures, 'freedos', 'kernel.sys'), os.path.join(driveC, 'kernel.sys'))
	# 4. 4dos
	os.symlink(res_path('4dos'), os.path.join(driveC, '4dos'))
	# 5. foxrun
	os.symlink(res_path('foxrun'), os.path.join(driveC, 'foxrun'))
	# 6. tmp
	mkdir_p(os.path.join(driveC, 'tmp'))

	# Drive D initially points to freedos
	driveD = os.path.join(drivesDir, 'd')
	os.symlink(os.path.join(dosemures, 'freedos'), driveD)

	# Drive E points to database path
	driveE = os.path.join(drivesDir, 'e')
	os.symlink(get_conf('dbPath'), driveE)

	# Drive F points to dosemu's freedos
	driveF = os.path.join(drivesDir, 'f')
	os.symlink(os.path.join(dosemures, 'freedos'), driveF)

	# Copy app data
	util.copy_tree(res_path(appName), os.path.join(driveC, appName))

	# Create config.txt for app
	create_configtxt(os.path.join(driveC, appName, 'config.txt'))

	# Create batch file to run app
	create_runbat(os.path.join(driveC, 'run.bat'))

	# Create dosemurc
	dosemurc = os.path.join(workDir, 'dosemurc')
	f = open(dosemurc, 'w')
	f.write("$_cpu_emu = \"full\"\n")
	f.write("$_layout = \"us\"\n")
	f.write("$_sound = (off)\n")
	f.write("$_lpt1 = \"lpr -l -P %s\"\n" % get_conf('printerName'))
	f.write("$_hdimage = \"%s/*\"\n" % drivesDir)
	f.write("$_X_title = \"%s\"\n" % appName)
	f.write("$_X_title_show_appname = \"off\"\n")
	f.write("$_X_icon_name = \"%s\"\n" % appName)
	f.close()

	# Run dosemu
	cmdLine = [os.path.join(dosemures, 'bin', 'dosemu.bin')]
	if options.textMode:
		cmdLine.append('-t')
	cmdLine = cmdLine + ['-f', dosemurc, 'run.bat']
	subprocess.call(cmdLine)

def get_version():
	ver = 'Unknown'
	versionFName = os.path.join(res_path(appName), 'VERSION')
	if os.path.exists(versionFName) and os.path.isfile(versionFName):
		f = open(versionFName)
		for line in f:
			if line.startswith('version:'):
				ver = line.split(' ')[1]
				break
		f.close()
	return ver

def process_flag_opts(opts):
	for opt in opts:
		if opt == '-v' or opt == '--version':
			print get_version()
			return -1
		if opt == '-t' or opt == '--text':
			options.textMode = True
			return 0
		if opt.startswith('-c'):
			ckey = opt[2:]
			if len(ckey) > 0:
				print get_conf(ckey)
			return -1
		if opt.startswith('-a'):
			ckey = opt[2:]
			if len(ckey) > 0:
				print get_app_conf(ckey)
			return -1
	return 0

def process_args(args):
	flagOpts = []
	options.appArgs = []
	for av in args:
		if av.startswith('-'):
			flagOpts.append(av)
		else:
			options.appArgs.append(av)

	if len(flagOpts) > 0:
		optsStat = process_flag_opts(flagOpts)
		if optsStat > 0:
			return optsStat
		if optsStat < 0:
			return -1

	if len(options.appArgs) == 0:
		options.appArgs.append(defaultEntryPoint)

	return 0

def init_config():
	config.set('dbPath', '/mnt/dbkorporat')
	config.set('printerName', 'epson-korporat')

	configFileDir = os.path.join(os.environ['HOME'], '.' + appName)
	configFileName = os.path.join(configFileDir, 'config.json')
	if not os.path.exists(configFileName):
		# Load old configuration
		oldConf = configlib.from_old_conf()
		config.merge(oldConf)

		# Write to file
		mkdir_p(configFileDir)
		f = open(configFileName, 'w')
		config.write_to_file(f)
		f.flush()
		f.close()
	else:
		f = open(configFileName, 'r')
		config.read_from_file(f)
		f.close()


if __name__ == '__main__':

	config = configlib.Config()
	init_config()

	if len(sys.argv) < 2:
		options.appArgs = [defaultEntryPoint]
	else:
		argStat = process_args(sys.argv[1:])
		if argStat > 0:
			exit(argStat)
		if argStat < 0:
			exit(0)

	launch()
